local bn_assets = require("BattleNetwork.Assets")

local GUARD_EFFECT_TEXTURE = bn_assets.load_texture("shield_impact.png")
local GUARD_EFFECT_ANIM_PATH = bn_assets.fetch_animation_path("shield_impact.animation")

local GUARD_AUDIO = bn_assets.load_audio("guard.ogg")

local TEXTURE = Resources.load_texture("Cube.png")

local function spawn_particle(texture, animation_path, field, tile)
	if not texture or not animation_path then
		return
	end

	local artifact = Artifact.new()
	artifact:set_texture(texture)
	artifact:sprite():set_layer(-5)

	local animation = artifact:animation()
	animation:load(animation_path)
	animation:set_state("DEFAULT")
	animation:on_complete(function()
		artifact:erase()
	end)

	field:spawn(artifact, tile)

	return artifact
end

local function spawn_impact_particle(user)
	local field = user:field()
	local tile = user:current_tile()
	local artifact = spawn_particle(GUARD_EFFECT_TEXTURE, GUARD_EFFECT_ANIM_PATH, field, tile)

	if not artifact then return end

	local sprite = artifact:sprite()
	local width = user:sprite():width()
	local height = user:height()

	sprite:set_offset(
		math.random(-width * .25, width * .25),
		math.random(-height * .75, -height * .25)
	)
end

function card_init(actor, props)
	local action = Action.new(actor, "CHARACTER_IDLE")
	action:override_animation_frames({ { 1, 30 } })

	action.on_execute_func = function(self, user)
		local cube = Obstacle.new(Team.Other)

		local field = user:field()

		local defense_rule = DefenseRule.new(DefensePriority.Body, DefenseOrder.CollisionOnly)

		defense_rule.defense_func = function(defense, _, _, defense_props)
			if defense_props.flags & Hit.PierceGuard ~= 0 then
				-- pierced
				cube:remove_defense_rule(defense_rule)
				cube:delete()
				return
			end

			defense:block_damage()

			if defense:impact_blocked() or defense_props.flags & Hit.Impact == 0 then
				-- non impact
				return
			end

			spawn_impact_particle(cube)
			Resources.play_audio(GUARD_AUDIO, AudioBehavior.NoOverlap)

			defense:block_impact()
		end

		cube:add_defense_rule(defense_rule)

		cube:set_facing(user:facing())
		cube:set_texture(TEXTURE)

		local anim = cube:animation()
		anim:load("PainCube.animation")
		anim:set_state("FALL")
		anim:set_playback(Playback.Loop)

		anim:apply(cube:sprite())

		anim:on_complete(function()
			anim:set_state("IDLE")
			anim:apply(cube:sprite())
			anim:set_playback(Playback.Loop)
		end)

		cube:set_health(200)
		cube:sprite():set_layer(-1)

		local countdown = 6000

		-- define cube collision hitprops
		local hit_props = HitProps.new(
			props.damage,
			props.hit_flags,
			props.element,
			user:context(),
			Drag.None
		)

		cube.on_collision_func = function(cube_self, other)
			local hitbox = Hitbox.new(cube:team())
			hitbox:set_hit_props(hit_props)

			field:spawn(hitbox, cube:current_tile())

			cube:delete()
		end

		-- upon passing the defense check
		cube.on_attack_func = function(self) end

		-- Can't move it, too heavy.
		cube.can_move_to_func = function(self, tile)
			return false
		end

		local shake = true

		cube.on_update_func = function(self)
			if self:elevation() > -2 then
				self:set_elevation(self:elevation() - 8)
				return
			end

			if shake then
				field:shake(8, 30)
				shake = false
			end

			local tile = cube:current_tile()
			if not tile then return cube:delete() end
			if not tile:is_walkable() then return cube:delete() end

			tile:attack_entities(cube)

			if countdown > 0 then countdown = countdown - 1 else cube:delete() end
		end

		cube.on_delete_func = function(self)
			local fx = Explosion.new()

			local fx_anim = fx:animation()

			fx_anim:on_complete(function()
				fx:erase()
			end)

			fx:sprite():set_layer(-2)
			field:spawn(fx, self:current_tile())

			cube:erase()
		end

		cube:set_elevation(102)

		cube:set_owner(user:team())

		field:spawn(cube, user:get_tile(user:facing(), 1))
	end
	return action
end
